<?php
if (! class_exists ( 'WC_Payment_Gateway' )) {
	return;
}
/**
 *
 * @since 3.0.0
 * @author PaymentPlugins
 * @package Stripe/Abstract
 *         
 */
abstract class WC_Payment_Gateway_Stripe extends WC_Payment_Gateway {

	private $admin_output = false;

	public $token_key;

	public $saved_method_key;

	public $payment_type_key;

	public $payment_intent_key;

	public $save_source_key;

	protected $tab_title;

	public $template_name;

	public $product_template;

	protected $checkout_error = false;

	/**
	 * Used to create an instance of a WC_Payment_Token
	 *
	 * @var string
	 */
	protected $token_type;

	/**
	 *
	 * @var WC_Stripe_Gateway
	 */
	protected $gateway;

	/**
	 *
	 * @var WP_Error
	 */
	protected $wp_error;

	/**
	 *
	 * @var string
	 */
	protected $payment_method_token = null;

	protected $new_source_token = null;

	/**
	 * Is the payment method synchronous or asynchronous
	 *
	 * @var bool
	 */
	public $synchronous = true;

	protected $post_payment_processes = [];

	/**
	 *
	 * @var bool
	 */
	public $processing_payment = false;

	public function __construct() {
		$this->token_key = $this->id . '_token_key';
		$this->saved_method_key = $this->id . '_saved_method_key';
		$this->save_source_key = $this->id . '_save_source_key';
		$this->payment_type_key = $this->id . '_payment_type_key';
		$this->payment_intent_key = $this->id . '_payment_intent_key';
		$this->has_fields = true;
		$this->init_form_fields ();
		$this->init_settings ();
		$this->title = $this->get_option ( 'title_text' );
		$this->description = $this->get_option ( 'description' );
		$this->hooks ();
		$this->init_supports ();
		$this->gateway = new WC_Stripe_Gateway ();
	}

	public function hooks() {
		add_filter ( 'wc_stripe_settings_nav_tabs', array( 
				$this, 'admin_nav_tab' 
		) );
		add_action ( 'woocommerce_settings_checkout_' . $this->id, [ 
				$this, 'enqueue_admin_scripts' 
		] );
		add_action ( 'woocommerce_update_options_payment_gateways_' . $this->id, [ 
				$this, 'process_admin_options' 
		] );
		add_filter ( 'woocommerce_payment_methods_list_item', [ 
				$this, 'payment_methods_list_item' 
		], 10, 2 );
		add_action ( 'wc_stripe_payment_token_deleted_' . $this->id, [ 
				$this, 'delete_payment_method' 
		], 10, 2 );
		add_filter ( 'woocommerce_subscription_payment_meta', [ 
				$this, 'subscription_payment_meta' 
		], 10, 2 );
		add_action ( 'woocommerce_scheduled_subscription_payment_' . $this->id, [ 
				$this, 'scheduled_subscription_payment' 
		], 10, 2 );
		if (! is_admin ()) {
			add_action ( 'woocommerce_subscriptions_pre_update_payment_method', [ 
					$this, 
					'subscriptions_pre_update_payment_method' 
			], 10, 3 );
			add_action ( 'woocommerce_subscription_payment_method_updated_to_' . $this->id, [ 
					$this, 
					'subscription_payment_method_updated' 
			] );
		}
		add_action ( 'wc_pre_orders_process_pre_order_completion_payment_' . $this->id, array( 
				$this, 'process_pre_order_payment' 
		) );
	}

	public function init_supports() {
		$this->supports = array( 'tokenization', 'products', 
				'subscriptions', 'add_payment_method', 
				'subscription_cancellation', 
				'multiple_subscriptions', 
				'subscription_amount_changes', 
				'subscription_date_changes', 
				'default_credit_card_form', 'refunds', 
				'pre-orders', 
				'subscription_payment_method_change_admin', 
				'subscription_reactivation', 
				'subscription_suspension', 
				'subscription_payment_method_change_customer' 
		);
	}

	public function admin_nav_tab($tabs) {
		$tabs[ $this->id ] = $this->tab_title;
		return $tabs;
	}

	public function init_form_fields() {
		$this->form_fields = include wc_stripe ()->plugin_path () . 'includes/gateways/settings/' . str_replace ( [ 
				'stripe_', '_' 
		], [ '', '-' 
		], $this->id ) . '-settings.php';
		$this->form_fields = apply_filters ( 'wc_stripe_form_fields_' . $this->id, $this->form_fields );
	}

	public function admin_options() {
		if ($this->admin_output) {
			return;
		}
		$this->output_settings_nav ();
		printf ( '<input type="hidden" id="wc_stripe_prefix" name="wc_stripe_prefix" value="%1$s"/>', $this->get_prefix () );
		echo '<div class="wc-stripe-settings-container ' . $this->id . '">';
		parent::admin_options ();
		echo '</div>';
		$this->admin_output = true;
	}

	public function output_settings_nav() {
		include wc_stripe ()->plugin_path () . 'includes/admin/views/html-settings-nav.php';
	}

	public function get_prefix() {
		return $this->plugin_id . $this->id . '_';
	}

	public function generate_multiselect_html($key, $data) {
		$value = ( array ) $this->get_option ( $key, array() );
		$data[ 'options' ] = array_merge ( array_flip ( $value ), $data[ 'options' ] );
		return parent::generate_multiselect_html ( $key, $data );
	}

	public function is_active($key) {
		return $this->get_option ( $key ) === 'yes';
	}

	public function get_method_formats() {
		$class_name = 'WC_Payment_Token_' . $this->token_type;
		if (class_exists ( $class_name )) {
			/**
			 *
			 * @var WC_Payment_Token_Stripe
			 */
			$token = new $class_name ();
			return $token->get_formats ();
		}
		return array();
	}

	public function enqueue_admin_scripts() {}

	public function payment_fields() {
		$this->enqueue_frontend_scripts ();
		wc_stripe_token_field ( $this );
		wc_stripe_payment_intent_field ( $this );
		wc_stripe_get_template ( 'checkout/stripe-payment-method.php', array( 
				'gateway' => $this, 
				'tokens' => is_add_payment_method_page () ? null : $this->get_tokens () 
		) );
	}

	/**
	 * Output the product payment fields.
	 */
	public function product_fields() {
		global $product;
		$this->enqueue_frontend_scripts ();
		wc_stripe_token_field ( $this );
		wc_stripe_get_template ( 'product/' . $this->template_name, array( 
				'gateway' => $this, 'product' => $product 
		) );
	}

	public function cart_fields() {
		$this->enqueue_frontend_scripts ();
		wc_stripe_token_field ( $this );
		wc_stripe_get_template ( 'cart/' . $this->template_name, array( 
				'gateway' => $this 
		) );
	}

	/**
	 * Enqueue scripts needed by the gateway on the frontend of the WC shop.
	 *
	 * @param WC_Stripe_Frontend_Scripts $scripts        	
	 */
	public function enqueue_frontend_scripts() {
		global $wp;
		if (is_add_payment_method_page ()) {
			$this->enqueue_add_payment_method_scripts ( wc_stripe ()->scripts () );
		}
		if (is_checkout ()) {
			$this->enqueue_checkout_scripts ( wc_stripe ()->scripts () );
		}
		if (is_cart ()) {
			$this->enqueue_cart_scripts ( wc_stripe ()->scripts () );
		}
		if (is_product ()) {
			$this->enqueue_product_scripts ( wc_stripe ()->scripts () );
		}
		if (! empty ( wc_stripe ()->scripts ()->enqueued_scripts )) {
			wc_stripe ()->scripts ()->enqueue_script ( 'form-handler' );
			wp_enqueue_style ( wc_stripe ()->scripts ()->prefix . 'styles', wc_stripe ()->assets_url () . 'css/stripe.css', array(), wc_stripe ()->version () );
		}
	}

	/**
	 * Enqueue scripts needed by the gateway on the checkout page.
	 *
	 * @param WC_Stripe_Frontend_Scripts $scripts        	
	 */
	public function enqueue_checkout_scripts($scripts) {}

	/**
	 * Enqueue scripts needed by the gateway on the add payment method page.
	 *
	 * @param WC_Stripe_Frontend_Scripts $scripts        	
	 */
	public function enqueue_add_payment_method_scripts($scripts) {
		$this->enqueue_checkout_scripts ( $scripts );
	}

	/**
	 * Enqueue scripts needed by the gateway on the cart page.
	 *
	 * @param WC_Stripe_Frontend_Scripts $scripts        	
	 */
	public function enqueue_cart_scripts($scripts) {}

	/**
	 * Enqueue scripts needed by the gateway on the product page.
	 *
	 * @param WC_Stripe_Frontend_Scripts $scripts        	
	 */
	public function enqueue_product_scripts($scripts) {}

	/**
	 *
	 * @return array
	 */
	public function get_localized_params() {
		return array( 'gateway_id' => $this->id, 
				'api_key' => wc_stripe_get_publishable_key (), 
				'saved_method_selector' => '[name="' . $this->saved_method_key . '"]', 
				'token_selector' => '[name="' . $this->token_key . '"]', 
				'payment_intent_selector' => '[name="' . $this->payment_intent_key . '"]', 
				'messages' => array( 
						'terms' => __ ( 'Please read and accept the terms and conditions to proceed with your order.', 'woocommerce' ) 
				), 
				'routes' => array( 
						'setup_intent' => wc_stripe ()->rest_api->payment_intent->rest_url ( 'setup-intent' ), 
						'add_to_cart' => wc_stripe ()->rest_api->cart->rest_url ( 'add-to-cart' ), 
						'cart_calculation' => wc_stripe ()->rest_api->cart->rest_url ( 'cart-calculation' ), 
						'shipping_method' => wc_stripe ()->rest_api->cart->rest_url ( 'shipping-method' ), 
						'shipping_address' => wc_stripe ()->rest_api->cart->rest_url ( 'shipping-address' ), 
						'checkout' => wc_stripe ()->rest_api->checkout->rest_url ( 'checkout' ) 
				), 
				'rest_nonce' => wp_create_nonce ( 'wp_rest' ), 
				'needs_shipping' => $this->get_needs_shipping (), 
				'banner_enabled' => $this->banner_checkout_enabled (), 
				'currency' => get_woocommerce_currency (), 
				'total_label' => __ ( 'Total', 'woo-stripe-payment' ), 
				'country_code' => wc_get_base_location ()[ 'country' ] 
		);
	}

	/**
	 * Returns the payment method the customer wants to use.
	 * This can be a saved payment method
	 * or a new payment method.
	 */
	protected function get_payment_method_from_request() {
		if ($this->use_saved_source ()) {
			return $this->get_saved_source_id ();
		}
		return $this->get_new_source_token ();
	}

	/**
	 * Save the Stripe data to the order.
	 *
	 * @param WC_Order $order        	
	 * @param \Stripe\Charge $charge        	
	 */
	public function save_order_meta($order, $charge) {
		/**
		 *
		 * @var WC_Payment_Token_Stripe $token
		 */
		$token = $this->get_payment_token ( $this->get_payment_method_from_charge ( $charge ), $charge->payment_method_details );
		$order->set_transaction_id ( $charge->id );
		$order->set_payment_method_title ( $token->get_payment_method_title () );
		$order->update_meta_data ( '_wc_stripe_mode', wc_stripe_mode () );
		$order->update_meta_data ( '_wc_stripe_charge_status', $charge->status );
		$order->update_meta_data ( '_payment_method_token', $token->get_token () );
		$order->save ();
		
		// if WCS is active and there are subscriptions in the order, save meta data
		if (wcs_stripe_active () && wcs_order_contains_subscription ( $order )) {
			foreach ( wcs_get_subscriptions_for_order ( $order ) as $subscription ) {
				$subscription->set_transaction_id ( $charge->id );
				$subscription->set_payment_method_title ( $token->get_payment_method_title () );
				$subscription->update_meta_data ( '_wc_stripe_mode', wc_stripe_mode () );
				$subscription->update_meta_data ( '_wc_stripe_charge_status', $charge->status );
				$subscription->update_meta_data ( '_payment_method_token', $token->get_token () );
				$subscription->update_meta_data ( '_wc_stripe_customer', wc_stripe_get_customer_id ( $order->get_user_id () ) );
				$subscription->save ();
			}
		}
	}

	/**
	 * Given a charge object, return the ID of the payment method used for the charge.
	 *
	 * @since 3.0.6
	 * @param \Stripe\Charge $charge        	
	 */
	public abstract function get_payment_method_from_charge($charge);

	/**
	 *
	 * {@inheritDoc}
	 *
	 * @see WC_Payment_Gateway::add_payment_method()
	 */
	public function add_payment_method() {
		$customer_id = wc_stripe_get_customer_id ();
		try {
			if (! is_user_logged_in ()) {
				throw new Exception ( __ ( 'User must be logged in.', 'woo-stripe-payment' ) );
			}
			if (empty ( $customer_id )) {
				$customer_id = $this->create_customer ( get_current_user_id () );
			}
			
			$result = $this->create_payment_method ( $this->get_new_source_token (), $customer_id );
			
			if (is_wp_error ( $result )) {
				wc_add_notice ( sprintf ( __ ( 'Error saving your payment method. Reason: %s', 'woo-stripe-payment' ), $result->get_error_message () ), 'error' );
				return array( 'result' => 'error' 
				);
			}
			$result->set_user_id ( get_current_user_id () );
			$result->save ();
			WC_Payment_Tokens::set_users_default ( get_current_user_id (), $result->get_id () );
			
			do_action ( 'wc_stripe_add_payment_method_success', $result );
			
			return array( 'result' => 'success', 
					'redirect' => wc_get_account_endpoint_url ( 'payment-methods' ) 
			);
		} catch ( \Stripe\Error\Base $e ) {
			wc_add_notice ( sprintf ( __ ( 'Error saving payment method. Reason: %s', 'woo-stripe-payment' ), $e->getMessage () ), 'error' );
			return array( 'result' => 'error' 
			);
		}
	}

	public function process_refund($order_id, $amount = null, $reason = '') {
		$order = wc_get_order ( $order_id );
		$amount_in_cents = wc_stripe_add_number_precision ( $amount );
		$charge = $order->get_transaction_id ();
		try {
			if (empty ( $charge )) {
				throw new Exception ( __ ( 'Transaction Id cannot be empty.', 'woo-stripe-payment' ) );
			}
			$result = $this->gateway->refund ( array( 
					'charge' => $charge, 
					'amount' => $amount_in_cents 
			), wc_stripe_order_mode ( $order ) );
			if (! is_wp_error ( $result )) {
				$order->add_order_note ( sprintf ( __ ( 'Order refunded in Stripe. Amount: %s', 'woo-stripe-payment' ), wc_price ( $amount, array( 
						'currency' => $order->get_currency () 
				) ) ) );
				return true;
			}
			return $result;
		} catch ( Exception $e ) {
			return new WP_Error ( 'refund-error', $e->getMessage () );
		}
	}

	/**
	 * Captures the charge for the Stripe order.
	 *
	 * @param float $amount        	
	 * @param WC_Order $order        	
	 */
	public function capture_charge($amount, $order) {}

	/**
	 * Void the Stripe charge.
	 *
	 * @param WC_Order $order        	
	 */
	public function void_charge($order) {}

	/**
	 * Return the \Stripe\Charge object
	 *
	 * @param String $charge_id        	
	 * @param String $mode        	
	 * @return WP_Error|\Stripe\Charge
	 */
	public function retrieve_charge($charge_id, $mode = '') {
		return $this->gateway->get_charge ( $charge_id, $mode );
	}

	/**
	 *
	 * @param string $method_id        	
	 * @param \Stripe\Card|array $method_details        	
	 */
	public function get_payment_token($method_id, $method_details) {
		$class_name = 'WC_Payment_Token_' . $this->token_type;
		if (class_exists ( $class_name )) {
			/**
			 *
			 * @var WC_Payment_Token_Stripe $token
			 */
			$token = new $class_name ( '', $method_details );
			$token->details_to_props ( $method_details );
			$token->set_token ( $method_id );
			$token->set_gateway_id ( $this->id );
			$token->set_format ( $this->get_option ( 'method_format' ) );
			$token->set_environment ( wc_stripe_mode () );
			return $token;
		}
	}

	/**
	 * Return a failed order response.
	 *
	 * @return array
	 */
	public function order_error() {
		wc_stripe_set_checkout_error ();
		return array( 'result' => 'failure' 
		);
	}

	/**
	 *
	 * @param WC_Order $order        	
	 */
	protected function get_order_identifier($order) {
		return sprintf ( '%s%s%s', $this->get_option ( 'order_prefix' ), $order->get_order_number (), $this->get_option ( 'order_suffix' ) );
	}

	/**
	 *
	 * @param WC_Order $order        	
	 */
	protected function get_name_from_order($order, $type) {
		if ($type === 'billing') {
			return sprintf ( '%s %s', $order->get_billing_first_name (), $order->get_billing_last_name () );
		} else {
			return sprintf ( '%s %s', $order->get_shipping_first_name (), $order->get_shipping_last_name () );
		}
	}

	/**
	 * Return the payment source the customer has chosen to use.
	 * This can be a saved source
	 * or a one time use source.
	 */
	public function get_payment_source() {
		if ($this->use_saved_source ()) {
			return $this->get_saved_source_id ();
		} else {
			if ($this->payment_method_token) {
				return $this->payment_method_token;
			}
			return $this->get_new_source_token ();
		}
	}

	public function get_payment_intent_id() {
		return ! empty ( $_POST[ $this->payment_intent_key ] ) ? $_POST[ $this->payment_intent_key ] : '';
	}

	/**
	 * Return true of the customer is using a saved payment method.
	 */
	public function use_saved_source() {
		return ( ! empty ( $_POST[ $this->payment_type_key ] ) && $_POST[ $this->payment_type_key ] === 'saved' ) || $this->payment_method_token;
	}

	/**
	 *
	 * @deprecated
	 *
	 */
	public function get_new_source_id() {
		return $this->get_new_source_token ();
	}

	public function get_new_source_token() {
		return null != $this->new_source_token ? $this->new_source_token : ( ! empty ( $_POST[ $this->token_key ] ) ? $_POST[ $this->token_key ] : '' );
	}

	public function get_saved_source_id() {
		return ! empty ( $_POST[ $this->saved_method_key ] ) ? $_POST[ $this->saved_method_key ] : $this->payment_method_token;
	}

	/**
	 * Create a customer in the stripe gateway.
	 *
	 * @param int $user_id        	
	 */
	public function create_customer($user_id) {
		$customer = WC ()->customer;
		$customer_id = $this->gateway->create_customer ( array( 
				'email' => $customer->get_email (), 
				'name' => sprintf ( '%s %s', $customer->get_first_name (), $customer->get_last_name () ), 
				'phone' => $customer->get_billing_phone () 
		) );
		if (! is_wp_error ( $customer )) {
			wc_stripe_save_customer ( $customer_id, $user_id );
			return $customer->id;
		} else {
			return $customer;
		}
	}

	/**
	 * Creates a payment method in Stripe.
	 *
	 * @param string $id
	 *        	payment method id
	 * @param string $customer_id
	 *        	WC Stripe customer ID
	 * @return WC_Payment_Token_Stripe|WP_Error
	 */
	abstract public function create_payment_method($id, $customer_id);

	/**
	 *
	 * @param array $item        	
	 * @param WC_Payment_Token_Stripe_CC $payment_token        	
	 */
	public function payment_methods_list_item($item, $payment_token) {
		if ($payment_token->get_type () === $this->token_type) {
			$item[ 'method' ][ 'last4' ] = $payment_token->get_last4 ();
			$item[ 'method' ][ 'brand' ] = ucfirst ( $payment_token->get_brand () );
			if ($payment_token->has_expiration ()) {
				$item[ 'expires' ] = sprintf ( '%s / %s', $payment_token->get_exp_month (), $payment_token->get_exp_year () );
			} else {
				$item[ 'expires' ] = __ ( 'n/a', 'woo-stripe-payment' );
			}
			$item[ 'wc_stripe_method' ] = true;
		}
		return $item;
	}

	/**
	 *
	 * @param string $token_id        	
	 * @param WC_Payment_Token_Stripe $token        	
	 */
	public function delete_payment_method($token_id, $token) {}

	public function saved_payment_methods($tokens = array()) {
		wc_stripe_get_template ( 'payment-methods.php', array( 
				'tokens' => $tokens, 'gateway' => $this 
		) );
	}

	public function get_new_method_label() {
		return __ ( 'New Card', 'woo-stripe-payment' );
	}

	public function get_saved_methods_label() {
		return __ ( 'Saved Cards', 'woo-stripe-payment' );
	}

	/**
	 * Return true if shipping is needed.
	 * Shipping is based on things like if the cart or product needs shipping.
	 *
	 * @return bool
	 */
	public function get_needs_shipping() {
		if (is_checkout () || is_cart ()) {
			global $wp;
			if (wcs_stripe_active () && WC_Subscriptions_Change_Payment_Gateway::$is_request_to_change_payment) {
				return false;
			}
			// return false if this is the order pay page. Gateways that have payment sheets don't need
			// to make any changes to the order.
			if (! empty ( $wp->query_vars[ 'order-pay' ] )) {
				return false;
			}
			return WC ()->cart->needs_shipping ();
		}
		if (is_product ()) {
			global $product;
			return is_a ( $product, 'WC_Product' ) && $product->needs_shipping ();
		}
	}

	/**
	 * Return true of the payment method should be saved.
	 *
	 * @param WC_Order $order        	
	 * @return bool
	 */
	public function should_save_payment_method($order) {
		if (! $this->use_saved_source ()) {
			if (wcs_stripe_active ()) {
				if (wcs_order_contains_subscription ( $order )) {
					return true;
				}
				if (wcs_cart_contains_renewal ()) {
					return true;
				}
			}
			if (! empty ( $_POST[ $this->save_source_key ] )) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Returns true if the save payment method checkbox can be displayed.
	 *
	 * @return boolean
	 */
	public function show_save_source() {
		global $wp;
		if (wcs_stripe_active ()) {
			if (WC_Subscriptions_Cart::cart_contains_subscription ()) {
				return false;
			}
			if (WC_Subscriptions_Change_Payment_Gateway::$is_request_to_change_payment) {
				return false;
			}
			if (wcs_cart_contains_renewal ()) {
				return false;
			}
		}
		return is_user_logged_in () && is_checkout () && $this->is_active ( 'save_card_enabled' );
	}

	/**
	 * Returns a formatted array of items for display in the payment gateway's payment sheet.
	 *
	 * @param bool $encode        	
	 * @return []
	 */
	public function get_display_items($encode = false) {}

	/**
	 * Returns a formatted array of shipping methods for display in the payment gateway's
	 * payment sheet.
	 *
	 * @param bool $encode        	
	 * @return []
	 */
	public function get_shipping_methods($encode = false) {}

	/**
	 * Return true if product page checkout is enabled for this gateway
	 *
	 * @return bool
	 */
	public function product_checkout_enabled() {
		return in_array ( 'product', $this->get_option ( 'payment_sections', [] ) );
	}

	/**
	 * Return true if cart page checkout is enabled for this gateway
	 *
	 * @return bool
	 */
	public function cart_checkout_enabled() {
		return in_array ( 'cart', $this->get_option ( 'payment_sections', [] ) );
	}

	/**
	 * Return true if checkout page banner is enabled for this gateway
	 *
	 * @return bool
	 */
	public function banner_checkout_enabled() {
		global $wp;
		return empty ( $wp->query_vars[ 'order-pay' ] ) && $this->supports ( 'wc_stripe_banner_checkout' ) && in_array ( 'checkout_banner', $this->get_option ( 'payment_sections', [] ) );
	}

	/**
	 * Decorate the response with data specific to the gateway.
	 *
	 * @param [] $data        	
	 */
	public function add_to_cart_response($data) {
		return $data;
	}

	/**
	 * Decorate the update shipping method reponse with data.
	 *
	 * @param [] $data        	
	 */
	public function get_update_shipping_method_response($data) {
		return $data;
	}

	/**
	 * Decorate the update shipping address respond with data.
	 *
	 * @param [] $data        	
	 */
	public function get_update_shipping_address_response($data) {
		return apply_filters ( 'wc_stripe_update_shipping_address_response', $data );
	}

	public function generate_button_demo_html($key, $data) {
		$field_key = $this->get_field_key ( $key );
		$data = wp_parse_args ( $data, array( 
				'title' => '', 'class' => '', 'style' => '', 
				'description' => '', 'desc_tip' => false, 
				'id' => 'wc-stripe-button-demo' 
		) );
		ob_start ();
		include wc_stripe ()->plugin_path () . 'includes/admin/views/html-button-demo.php';
		return ob_get_clean ();
	}

	public function generate_stripe_button_html($key, $data) {
		$field_key = $this->get_field_key ( $key );
		$data = wp_parse_args ( $data, array( 
				'title' => '', 'class' => '', 'style' => '', 
				'description' => '', 'desc_tip' => false, 
				'id' => 'wc-stripe-button_' . $key 
		) );
		ob_start ();
		include wc_stripe ()->plugin_path () . 'includes/admin/views/html-button.php';
		return ob_get_clean ();
	}

	public function generate_description_html($key, $data) {
		$field_key = $this->get_field_key ( $key );
		$data = wp_parse_args ( $data, array( 
				'class' => '', 'style' => '', 
				'description' => '' 
		) );
		ob_start ();
		include wc_stripe ()->plugin_path () . 'includes/admin/views/html-description.php';
		return ob_get_clean ();
	}

	/**
	 * Returns an array of metadata for the transaction.
	 *
	 * @param WC_Order $order        	
	 */
	protected function get_order_meta_data($order) {
		$meta_data = [ 'gateway_id' => $this->id, 
				'order_id' => $order->get_order_number (), 
				'user_id' => $order->get_user_id (), 
				'customer_id' => wc_stripe_get_customer_id ( $order->get_user_id () ), 
				'ip_address' => $order->get_customer_ip_address (), 
				'user_agent' => isset ( $_SERVER[ 'HTTP_USER_AGENT' ] ) ? $_SERVER[ 'HTTP_USER_AGENT' ] : 'unavailable', 
				'partner' => 'PaymentPlugins' 
		];
		foreach ( $order->get_items ( 'line_item' ) as $item ) {
			/**
			 *
			 * @var WC_Order_Item_Product $item
			 */
			$meta_data[ 'product_' . $item->get_product_id () ] = sprintf ( '%s x %s', $item->get_name (), $item->get_quantity () );
		}
		return apply_filters ( 'wc_stripe_order_meta_data', $meta_data, $order );
	}

	/**
	 * Save the customer's payment method.
	 * If the payment method has already been saved to the customer
	 * the simply return true.
	 *
	 * @param string $id        	
	 * @param WC_Order $order        	
	 * @return WP_Error|bool
	 */
	public function save_payment_method($id, $order) {
		$token = $this->create_payment_method ( $id, wc_stripe_get_customer_id ( $order->get_user_id () ) );
		if (is_wp_error ( $token )) {
			$this->wp_error = $token;
			$order->add_order_note ( sprintf ( __ ( 'Attemp to save payment method failed. Reason: %s', 'woo-stripe-payment' ), $token->get_error_message () ) );
			return $token;
		}
		$token->set_user_id ( $order->get_user_id () );
		$token->save ();
		
		// set token value so it can be used for other processes.
		$this->payment_method_token = $token->get_token ();
		return true;
	}

	/**
	 * Set an error on the order.
	 * This error is used on the frontend to alert customer's to a failed payment method save.
	 *
	 * @param WC_Order $order        	
	 * @param WP_Error $error        	
	 */
	protected function set_payment_save_error($order, $error) {
		if (wcs_stripe_active () && wcs_order_contains_subscription ( $order )) {
			$message = __ ( 'We were not able to save your payment method. To prevent billing issues with your subscription, please add a payment method to the subscription.', 'woo-stripe-payment' );
		} else {
			$message = sprintf ( __ ( 'We were not able to save your payment method. Reason: %s', 'woo-stripe-payment' ), $error->get_error_message () );
		}
		$order->update_meta_data ( '_wc_stripe_order_error', $message );
		$order->save ();
	}

	/**
	 *
	 * @param string $token_id        	
	 * @param int $user_id        	
	 * @return null|WC_Payment_Token_Stripe_CC
	 */
	public function get_token($token_id, $user_id) {
		$tokens = WC_Payment_Tokens::get_customer_tokens ( $user_id, $this->id );
		foreach ( $tokens as $token ) {
			if ($token_id === $token->get_token ()) {
				return $token;
			}
		}
		return null;
	}

	/**
	 *
	 * @param array $payment_meta        	
	 * @param WC_Subscription $subscription        	
	 */
	public function subscription_payment_meta($payment_meta, $subscription) {
		$payment_meta[ $this->id ] = array( 
				'post_meta' => array( 
						'_payment_method_token' => array( 
								'value' => $subscription->get_meta ( '_payment_method_token', true ), 
								'label' => __ ( 'Payment Method Token', 'woo-stripe-payment' ) 
						) 
				) 
		);
		return $payment_meta;
	}

	/**
	 *
	 * @param float $amount        	
	 * @param WC_Order $order        	
	 */
	public function scheduled_subscription_payment($amount, $order) {}

	/**
	 *
	 * @param WC_Subscription $subscription        	
	 * @param string $new_payment_method        	
	 * @param string $old_payment_method        	
	 */
	public function subscriptions_pre_update_payment_method($subscription, $new_payment_method, $old_payment_method) {
		if ($new_payment_method === $this->id) {
			// customer is using a new payment method so we need to save it.
			if (! $this->use_saved_source ()) {
				$result = $this->save_payment_method ( $this->get_new_source_token (), $subscription );
				if (is_wp_error ( $result )) {
					wc_add_notice ( sprintf ( __ ( 'Error saving payment method for subscription. Reason: %s', 'woo-stripe-payment' ), $result->get_error_message () ), 'error' );
					// redirect to change payment method page.
					wp_safe_redirect ( $subscription->get_view_order_url () );
				}
			} else {
				// set the payment method token so it can be used downstream.
				$this->payment_method_token = $this->get_payment_source ();
			}
		}
	}

	/**
	 * Update the subscription with the new meta data for the gateway.
	 * Save any new method the customer is trying to use.
	 *
	 * @param WC_Subscription $subscription        	
	 */
	public function subscription_payment_method_updated($subscription) {
		$token = $this->get_token ( $this->payment_method_token, $subscription->get_user_id () );
		
		// update the meta data needed by the gateway to process a subscription payment.
		$subscription->update_meta_data ( '_payment_method_token', $this->payment_method_token );
		$subscription->set_payment_method_title ( $token->get_payment_method_title () );
		$subscription->save ();
	}

	/**
	 * Return true if this request is to change the payment method of a WC Subscription.
	 *
	 * @return bool
	 */
	public function is_change_payment_method_request() {
		return wcs_stripe_active () && did_action ( 'woocommerce_subscriptions_pre_update_payment_method' );
	}

	/**
	 * Sets the ID of a payment token.
	 *
	 * @param string $id        	
	 */
	public function set_payment_method_token($id) {
		$this->payment_method_token = $id;
	}

	public function set_new_source_token($token) {
		$this->new_source_token = $token;
	}

	/**
	 *
	 * @param WC_Order $order        	
	 */
	public function get_order_description($order) {
		return sprintf ( __ ( 'Order %s from %s', 'woo-stripe-payment' ), $order->get_order_number (), get_bloginfo ( 'name' ) );
	}

	public function get_custom_attribute_html($attribs) {
		if (! empty ( $attribs[ 'custom_attributes' ] ) && is_array ( $attribs[ 'custom_attributes' ] )) {
			foreach ( $attribs[ 'custom_attributes' ] as $k => $v ) {
				if (is_array ( $v )) {
					$attribs[ 'custom_attributes' ][ $k ] = htmlspecialchars ( wp_json_encode ( $v ) );
				}
			}
		}
		return parent::get_custom_attribute_html ( $attribs );
	}

	/**
	 *
	 * @param WC_Order $order        	
	 */
	public function process_zero_total_order($order) {
		// save payment method if necessary
		if ($this->should_save_payment_method ( $order )) {
			$result = $this->save_payment_method ( $this->get_new_source_token (), $order );
			if (is_wp_error ( $result )) {
				return $this->order_error ();
			}
		} else {
			$this->payment_method_token = $this->get_saved_source_id ();
		}
		$this->save_zero_total_meta ( $order );
		$order_status = $this->get_option ( 'order_status' );
		if ('capture' === $this->get_option ( 'charge_type' )) {
			$order->payment_complete ();
		} else {
			$order_status = $this->get_option ( 'order_status' );
			$order->update_status ( apply_filters ( 'wc_stripe_authorized_order_status', 'default' === $order_status ? 'processing' : $order_status, $order, $this ) );
		}
		WC ()->cart->empty_cart ();
		return array( 'result' => 'success', 
				'redirect' => $order->get_checkout_order_received_url () 
		);
	}

	public function process_pre_order($order) {
		// maybe save payment method
		if (! $this->use_saved_source ()) {
			$result = $this->save_payment_method ( $this->get_new_source_token (), $order );
			if (is_wp_error ( $result )) {
				return $this->order_error ();
			}
		} else {
			$this->payment_method_token = $this->get_saved_source_id ();
		}
		WC_Pre_Orders_Order::mark_order_as_pre_ordered ( $order );
		$this->save_zero_total_meta ( $order );
		return array( 'result' => 'success', 
				'redirect' => $order->get_checkout_order_received_url () 
		);
	}

	/**
	 *
	 * @param WC_Order $order        	
	 */
	protected function save_zero_total_meta($order) {
		$token = $this->get_token ( $this->payment_method_token, $order->get_user_id () );
		$order->set_payment_method_title ( $token->get_payment_method_title () );
		$order->update_meta_data ( '_wc_stripe_mode', wc_stripe_mode () );
		$order->update_meta_data ( '_payment_method_token', $token->get_token () );
		$order->save ();
		
		if (wcs_stripe_active () && wcs_order_contains_subscription ( $order )) {
			foreach ( wcs_get_subscriptions_for_order ( $order ) as $subscription ) {
				/**
				 *
				 * @var WC_Subscription $subscription
				 */
				$subscription->set_payment_method_title ( $token->get_payment_method_title () );
				$subscription->update_meta_data ( '_wc_stripe_mode', wc_stripe_mode () );
				$subscription->update_meta_data ( '_payment_method_token', $token->get_token () );
				$subscription->update_meta_data ( '_wc_stripe_customer', wc_stripe_get_customer_id ( $order->get_user_id () ) );
				$subscription->save ();
			}
		}
	}

	/**
	 * Pre orders can't be mixed with regular products.
	 *
	 * @param WC_Order $order        	
	 */
	protected function order_contains_pre_order($order) {
		return wc_stripe_pre_orders_active () && WC_Pre_Orders_Order::order_contains_pre_order ( $order );
	}

	/**
	 *
	 * @param WC_Order $order        	
	 * @return boolean
	 */
	protected function pre_order_requires_tokenization($order) {
		return WC_Pre_Orders_Order::order_requires_payment_tokenization ( $order );
	}

	/**
	 * Sets a lock on the order.
	 * Default behavior is a 2 minute lock.
	 *
	 * @param WC_Order|int $order        	
	 */
	public function set_order_lock($order) {
		set_transient ( '_stripe_lock_order_' . ( is_object ( $order ) ? $order->get_id () : $order ), apply_filters ( 'wc_stripe_set_order_lock', time () + 2 * MINUTE_IN_SECONDS ) );
	}

	/**
	 * Removes the lock on the order
	 *
	 * @param WC_Order|int $order        	
	 */
	public function release_order_lock($order) {
		delete_transient ( '_stripe_lock_order_' . ( is_object ( $order ) ? $order->get_id () : $order ) );
	}

	/**
	 * Returns true of the order has been locked.
	 * If the lock exists and is greater than current time
	 * method returns true;
	 *
	 * @param WC_Order|int $order        	
	 */
	public function has_order_lock($order) {
		$lock = get_transient ( '_stripe_lock_order_' . ( is_object ( $order ) ? $order->get_id () : $order ) );
		return $lock != false;
	}

	public function set_post_payment_process($callback) {
		$this->post_payment_processes[] = $callback;
	}

	/**
	 *
	 * @param WC_Order $order        	
	 * @param WC_Payment_Gateway_Stripe $gateway        	
	 */
	public function trigger_post_payment_processes($order, $gateway) {
		foreach ( $this->post_payment_processes as $callback ) {
			call_user_func_array ( $callback, func_get_args () );
		}
	}

	public function validate_payment_sections_field($key, $value) {
		if (empty ( $value )) {
			$value = [];
		}
		return $value;
	}
}