<?php
/**
 * 
 * @author PaymentPlugins
 * @package Stripe/Abstract
 *
 */
abstract class WC_Stripe_Settings_API extends WC_Settings_API {

	protected $tab_title;

	private $admin_output = false;

	public function __construct() {
		$this->init_form_fields ();
		$this->init_settings ();
		$this->hooks ();
	}

	public function hooks() {
		add_action ( 'wc_stripe_localize_' . $this->id . '_settings', array( 
				$this, 'localize_settings' 
		) );
	}

	public function admin_nav_tab($tabs) {
		$tabs[ $this->id ] = $this->tab_title;
		return $tabs;
	}

	public function admin_options() {
		if ($this->admin_output) {
			return;
		}
		$this->display_errors ();
		$this->output_settings_nav ();
		printf ( '<input type="hidden" id="wc_stripe_prefix" name="wc_stripe_prefix" value="%1$s"/>', $this->get_prefix () );
		echo '<div class="wc-stripe-settings-container ' . $this->id . '">';
		parent::admin_options ();
		echo '</div>';
		$this->admin_output = true;
	}

	public function output_settings_nav() {
		include wc_stripe ()->plugin_path () . 'includes/admin/views/html-settings-nav.php';
	}

	public function get_prefix() {
		return $this->plugin_id . $this->id . '_';
	}

	public function localize_settings() {
		return $this->settings;
	}

	public function get_custom_attribute_html($attribs) {
		if (! empty ( $attribs[ 'custom_attributes' ] ) && is_array ( $attribs[ 'custom_attributes' ] )) {
			foreach ( $attribs[ 'custom_attributes' ] as $k => $v ) {
				if (is_array ( $v )) {
					$attribs[ 'custom_attributes' ][ $k ] = htmlspecialchars ( wp_json_encode ( $v ) );
				}
			}
		}
		return parent::get_custom_attribute_html ( $attribs );
	}

	public function is_active($key) {
		return $this->get_option ( $key ) === 'yes';
	}

	public function generate_description_html($key, $data) {
		$field_key = $this->get_field_key ( $key );
		$data = wp_parse_args ( $data, array( 
				'class' => '', 'style' => '', 
				'description' => '' 
		) );
		ob_start ();
		include wc_stripe ()->plugin_path () . 'includes/admin/views/html-description.php';
		return ob_get_clean ();
	}

	public function generate_paragraph_html($key, $data) {
		$field_key = $this->get_field_key ( $key );
		$defaults = array( 'title' => '', 'label' => '', 
				'class' => '', 'css' => '', 'type' => 'text', 
				'desc_tip' => false, 'description' => '', 
				'custom_attributes' => array() 
		);
		$data = wp_parse_args ( $data, $defaults );
		if (! $data[ 'label' ]) {
			$data[ 'label' ] = $data[ 'title' ];
		}
		ob_start ();
		include wc_stripe ()->plugin_path () . 'includes/admin/views/html-paragraph.php';
		return ob_get_clean ();
	}

	public function generate_stripe_button_html($key, $data) {
		$field_key = $this->get_field_key ( $key );
		$data = wp_parse_args ( $data, array( 
				'title' => '', 'class' => '', 'style' => '', 
				'description' => '', 'desc_tip' => false, 
				'id' => 'wc-stripe-button_' . $key, 
				'disabled' => false, 'css' => '' 
		) );
		ob_start ();
		include wc_stripe ()->plugin_path () . 'includes/admin/views/html-button.php';
		return ob_get_clean ();
	}
}