<?php
class WC_Stripe_API_Settings extends WC_Stripe_Settings_API {

	public function __construct() {
		$this->id = 'stripe_api';
		$this->tab_title = __ ( 'API Settings', 'woo-stripe-payment' );
		parent::__construct ();
	}

	public function hooks() {
		parent::hooks ();
		add_action ( 'woocommerce_update_options_checkout_' . $this->id, array( 
				$this, 'process_admin_options' 
		) );
		add_filter ( 'wc_stripe_settings_nav_tabs', array( 
				$this, 'admin_nav_tab' 
		) );
		add_action ( 'woocommerce_settings_checkout_' . $this->id, array( 
				$this, 'admin_options' 
		) );
	}

	public function init_form_fields() {
		$this->form_fields = array( 
				'title' => array( 'type' => 'title', 
						'title' => __ ( 'API Settings', 'woo-stripe-payment' ) 
				), 
				'mode' => array( 'type' => 'select', 
						'title' => __ ( 'Mode', 'woo-stripe-payment' ), 
						'class' => 'wc-enhanced-select', 
						'options' => array( 
								'test' => __ ( 'Test', 'woo-stripe-payment' ), 
								'live' => __ ( 'Live', 'woo-stripe-payment' ) 
						), 'default' => 'test', 
						'desc_tip' => true, 
						'description' => __ ( 'The mode determines if you are processing test transaction or live transactions on your site. Test mode allows you to simulate payments so you can test your integration.', 'woo-stripe-payment' ) 
				), 
				'publishable_key_test' => array( 
						'title' => __ ( 'Test Publishable Key', 'woo-stripe-payment' ), 
						'type' => 'text', 'default' => '', 
						'desc_tip' => true, 
						'description' => __ ( 'Your publishable key is used to initialize Stripe assets.', 'woo-stripe-payment' ), 
						'custom_attributes' => array( 
								'data-show-if' => array( 
										'mode' => 'test' 
								) 
						) 
				), 
				'secret_key_test' => array( 
						'title' => __ ( 'Test Secret Key', 'woo-stripe-payment' ), 
						'type' => 'password', 
						'default' => '', 'desc_tip' => true, 
						'description' => __ ( 'Your secret key is used to authenticate requests to Stripe.', 'woo-stripe-payment' ), 
						'custom_attributes' => array( 
								'data-show-if' => array( 
										'mode' => 'test' 
								) 
						) 
				), 
				'publishable_key_live' => array( 
						'title' => __ ( 'Live Publishable Key', 'woo-stripe-payment' ), 
						'type' => 'text', 'default' => '', 
						'desc_tip' => true, 
						'description' => __ ( 'Your publishable key is used to initialize Stripe assets.', 'woo-stripe-payment' ), 
						'custom_attributes' => array( 
								'data-show-if' => array( 
										'mode' => 'live' 
								) 
						) 
				), 
				'secret_key_live' => array( 
						'title' => __ ( 'Live Secret Key', 'woo-stripe-payment' ), 
						'type' => 'password', 
						'default' => '', 'desc_tip' => true, 
						'description' => __ ( 'Your secret key is used to authenticate requests to Stripe.', 'woo-stripe-payment' ), 
						'custom_attributes' => array( 
								'data-show-if' => array( 
										'mode' => 'live' 
								) 
						) 
				), 
				'connection_test' => array( 
						'type' => 'stripe_button', 
						'title' => __ ( 'Connection Test', 'woo-stripe-payment' ), 
						'label' => __ ( 'Connection Test', 'woo-stripe-payment' ), 
						'class' => 'wc-stripe-connection-test button-secondary', 
						'description' => __ ( 'Click this button to perform a connection test. If successful, you have entered your API keys correctly.', 'woo-stripe-payment' ) 
				), 
				'webhook_button' => array( 
						'type' => 'stripe_button', 
						'title' => __ ( 'Generate Webhook', 'woo-stripe-payment' ), 
						'label' => __ ( 'Generate Webhook', 'woo-stripe-payment' ), 
						'class' => 'wc-stripe-create-webhook button-secondary', 
						'description' => __ ( 'Click this button to configure webhooks automatically.', 'woo-stripe-payment' ) 
				), 
				'webhook_url' => array( 
						'type' => 'paragraph', 
						'title' => __ ( 'Webhook url', 'woo-stripe-payment' ), 
						'class' => 'wc-stripe-webhook', 
						'text' => wc_stripe ()->rest_api->webhook->rest_url ( 'webhook' ), 
						'description' => sprintf ( __ ( '<strong>Important:</strong> the webhook url is called by Stripe when events occur in your account, like a source becomes chargeable. You must add this webhook to your Stripe Dashboard if you are using any of the local gateways. %sWebhook guide%s', 'woo-stripe-payment' ), '<a target="_blank" href="https://docs.paymentplugins.com/wc-stripe/config/#/webhooks?id=configure-webhooks">', '</a>' ) 
				), 
				'webhook_secret_live' => array( 
						'type' => 'password', 
						'title' => __ ( 'Live Webhook Secret', 'woo-stripe-payment' ), 
						'description' => sprintf ( __ ( 'The webhook secret is used to authenticate webhooks sent from Stripe. It ensures no 3rd party can send you events, pretending to be Stripe. %sWebhook guide%s', 'woo-stripe-payment' ), '<a target="_blank" href="https://docs.paymentplugins.com/wc-stripe/config/#/webhooks?id=configure-webhooks">', '</a>' ), 
						'custom_attributes' => array( 
								'data-show-if' => array( 
										'mode' => 'live' 
								) 
						) 
				), 
				'webhook_secret_test' => array( 
						'type' => 'password', 
						'title' => __ ( 'Test Webhook Secret', 'woo-stripe-payment' ), 
						'description' => sprintf ( __ ( 'The webhook secret is used to authenticate webhooks sent from Stripe. It ensures no 3rd party can send you events, pretending to be Stripe. %sWebhook guide%s', 'woo-stripe-payment' ), '<a target="_blank" href="https://docs.paymentplugins.com/wc-stripe/config/#/webhooks?id=configure-webhooks">', '</a>' ), 
						'custom_attributes' => array( 
								'data-show-if' => array( 
										'mode' => 'test' 
								) 
						) 
				), 
				'debug_log' => array( 
						'title' => __ ( 'Debug Log', 'woo-stripe-payment' ), 
						'type' => 'checkbox', 
						'desc_tip' => true, 
						'default' => 'yes', 
						'description' => __ ( 'When enabled, the plugin logs important errors and info that can help you troubleshoot potential issues.', 'woo-stripe-payment' ) 
				) 
		);
	}
	
	/*
	 * public function validate_publishable_key_test_field($key, $value) {
	 * if (! strstr ( $value, 'test' )) {
	 * throw new Exception ( __ ( 'You have entered an invalid publishable key for Test mode', 'woo-stripe-payment' ) );
	 * }
	 * return $value;
	 * }
	 *
	 * public function validate_publishable_key_live_field($key, $value) {
	 * if (! strstr ( $value, 'live' )) {
	 * throw new Exception ( __ ( 'You have entered an invalid publishable key for Live mode', 'woo-stripe-payment' ) );
	 * }
	 * return $value;
	 * }
	 */
}