<?php
if (! class_exists ( 'WC_Payment_Gateway_Local_Payment' )) {
	return;
}
/**
 *
 * @package Stripe/Gateways
 * @author PaymentPlugins
 *        
 */
class WC_Payment_Gateway_Stripe_Sofort extends WC_Payment_Gateway_Stripe_Local_Payment {

	public function __construct() {
		$this->synchronous = false;
		$this->local_payment_type = 'sofort';
		$this->currencies = [ 'EUR' 
		];
		$this->countries = [ 'AT', 'BE', 'DE', 'ES', 
				'IT', 'NL' 
		];
		$this->id = 'stripe_sofort';
		$this->tab_title = __ ( 'Sofort', 'woo-stripe-payment' );
		$this->template_name = 'local-payment.php';
		$this->token_type = 'Stripe_Local';
		$this->method_title = __ ( 'Sofort', 'woo-stripe-payment' );
		$this->method_description = __ ( 'Sofort gateway that integrates with your Stripe account.', 'woo-stripe-payment' );
		$this->icon = wc_stripe ()->assets_url ( 'img/sofort.svg' );
		$this->order_button_text = __ ( 'SOFORT', 'woo-stripe-payment' );
		parent::__construct ();
	}

	public function get_local_payment_settings() {
		return [ 
				'desc' => array( 'type' => 'description', 
						'description' => $this->get_payment_description()
				), 
				'enabled' => array( 
						'title' => __ ( 'Enabled', 'woo-stripe-payment' ), 
						'type' => 'checkbox', 
						'default' => 'no', 'value' => 'yes', 
						'desc_tip' => true, 
						'description' => __ ( 'If enabled, your site can accept Sofort payments through Stripe.', 'woo-stripe-payment' ) 
				), 
				'general_settings' => array( 
						'type' => 'title', 
						'title' => __ ( 'General Settings', 'woo-stripe-payment' ) 
				), 
				'title_text' => array( 'type' => 'text', 
						'title' => __ ( 'Title', 'woo-stripe-payment' ), 
						'default' => __ ( 'SOFORT', 'woo-stripe-payment' ), 
						'desc_tip' => true, 
						'description' => __ ( 'Title of the Sofort gateway' ) 
				), 
				'description' => array( 
						'title' => __ ( 'Description', 'woo-stripe-payment' ), 
						'type' => 'text', 'default' => '', 
						'description' => __ ( 'Leave blank if you don\'t want a description to show for the gateway.', 'woo-stripe-payment' ), 
						'desc_tip' => true 
				) 
		];
	}

	public function get_source_args($order) {
		return array_merge ( parent::get_source_args ( $order ), [ 
				'sofort' => [ 
						'country' => $order->get_billing_country () 
				] 
		] );
	}
}